Package: broadcast
Title: Broadcasted Array Operations Like 'NumPy'
Version: 0.1.9
Authors@R: person("Tony", "Wilkes", role = c("aut", "cre", "cph"),
                  email = "tony_a_wilkes@outlook.com",
                  comment = c(ORCID = "0000-0001-9498-8379"))
Description: Implements efficient 'NumPy'-like broadcasted operations for atomic and recursive arrays.
    In the context of operations involving 2 (or more) arrays,
    “broadcasting” (AKA singleton expansion) refers to efficiently recycling array dimensions,
    without making copies.
		Besides linking to 'Rcpp',
		'broadcast' does not use any external libraries in any way;
		'broadcast' was essentially made from scratch and can be installed out-of-the-box.
		The implementations available in 'broadcast' include, but are not limited to, the following.
		  1) Broadcasted element-wise operations on any 2 arrays;
		  they support a large set of
		  relational, arithmetic, Boolean, string, and bit-wise operations.
		  2) A faster, more memory efficient, and broadcasted abind-like function,
		  for binding arrays along an arbitrary dimension.
		  3) Broadcasted ifelse-like and apply-like functions.
		  4) Casting functions,
		  that cast subset-groups of an array to a new dimension, cast nested lists to dimensional lists, and vice-versa.
		  5) A few linear algebra functions for statistics.
		The functions in the 'broadcast' package strive to minimize computation time and memory usage
		(which is not just better for efficient computing, but also for the environment).
License: MPL-2.0
Encoding: UTF-8
LinkingTo: Rcpp
RoxygenNote: 7.3.2
Depends: R (>= 4.2.0)
Imports: Rcpp (>= 1.0.14), methods
Suggests: tinytest, abind, roxygen2
URL: https://github.com/tony-aw/broadcast,
        https://tony-aw.github.io/broadcast/
BugReports: https://github.com/tony-aw/broadcast/issues/
Language: en-gb
NeedsCompilation: yes
Packaged: 2026-05-29 14:41:31 UTC; Tony
Author: Tony Wilkes [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0001-9498-8379>)
Maintainer: Tony Wilkes <tony_a_wilkes@outlook.com>
Repository: CRAN
Date/Publication: 2026-05-29 15:50:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-29 16:22:54 UTC; unix
Archs: broadcast.so.dSYM
