# Generated by roxygen2: do not edit by hand

S3method("!=",broadcaster)
S3method("%%",broadcaster)
S3method("%/%",broadcaster)
S3method("&",broadcaster)
S3method("*",broadcaster)
S3method("+",broadcaster)
S3method("-",broadcaster)
S3method("/",broadcaster)
S3method("<",broadcaster)
S3method("<=",broadcaster)
S3method("==",broadcaster)
S3method(">",broadcaster)
S3method(">=",broadcaster)
S3method("[",broadcaster)
S3method("[[",broadcaster)
S3method("^",broadcaster)
S3method("|",broadcaster)
S3method(acast,default)
S3method(cast_dim2flat,default)
S3method(cast_dim2hier,default)
S3method(cast_hier2dim,default)
S3method(cast_shallow2atomic,default)
S3method(checkNA,default)
S3method(checkNULL,default)
S3method(dropnests,default)
S3method(format,broadcaster)
S3method(hier2dim,default)
S3method(hiernames2dimnames,default)
S3method(print,broadcaster)
export("bcr<-")
export("broadcaster<-")
export(acast)
export(array2vector)
export(as_bool)
export(as_chr)
export(as_cplx)
export(as_dbl)
export(as_int)
export(as_list)
export(as_num)
export(as_raw)
export(as_str)
export(bc.b)
export(bc.bit)
export(bc.cplx)
export(bc.d)
export(bc.i)
export(bc.list)
export(bc.raw)
export(bc.rel)
export(bc.str)
export(bc_dim)
export(bc_ifelse)
export(bc_strrep)
export(bcapply)
export(bcr)
export(bind_array)
export(broadcaster)
export(cast_dim2flat)
export(cast_dim2hier)
export(cast_hier2dim)
export(cast_shallow2atomic)
export(checkNA)
export(checkNULL)
export(cinv)
export(dropnests)
export(ecumprob)
export(hier2dim)
export(hiernames2dimnames)
export(lst.ndim)
export(mbroadcasters)
export(ndim)
export(rep_dim)
export(sd_lc)
export(undim)
export(vector2array)
exportMethods(bc.b)
exportMethods(bc.bit)
exportMethods(bc.cplx)
exportMethods(bc.d)
exportMethods(bc.i)
exportMethods(bc.list)
exportMethods(bc.raw)
exportMethods(bc.rel)
exportMethods(bc.str)
exportMethods(bc_ifelse)
exportMethods(bc_strrep)
exportMethods(bcapply)
importFrom(Rcpp,evalCpp)
importFrom(methods,setGeneric)
importFrom(methods,setMethod)
useDynLib(broadcast, .registration=TRUE)
